{

This unit is an addition to the small graphics library for VGA
adapters. It supports image manipulation for graphics display. Note that
no Assembler has been used in this module. It could obviously be made
faster by doing so, but it is done in Pascal for compatability across
multiple compilers (my requirement).

The routines contained in this module are as follows:

PROCEDURE flip(source,dest:Word);

  This procedure copies a screen from source to destination. This allows
  the developer to have more than one logical screen, and display the
  appropriate screen to the user.

PROCEDURE GetImage(x1,y1,x2,y2:Integer; VAR bitmap:pointer; needmem:Boolean);

  This procedure gets an image area from the screen as defined by
  X1,Y1 and X2,Y2 to a pointed to location. If the programmer wishes,
  NeedMem utilizes the routine to allocate the memory for the image.

PROCEDURE PutImage(x,y:Integer; bitmap:pointer; mode:Byte;where:Word);

  This procedure puts a previously saved (GetImage) image to the screen
  at X,Y pointed to by Bitmap. Where is the logical (or physical) screen
  to utilize. Mode is the mode for the placement: XorPut, AndPut, Normal
  Put, NoDistort, OrPut. Refer to a graphics tutorial if you are unclear
  on these modes.

PROCEDURE FlipImage(bitmap:pointer;VAR newpic:pointer);

  This procedure flips an image back to the screen. This allows for fast
  image display (as in a game, etc.).

PROCEDURE scaleBitmap(VAR bitmap;x,y:Word;x1,y1,x2,y2:Word);

  This procedure scales a bitmap stored previously from one size to
  another. x,y refer to the coordinates of the starting size, and
  x1,y1,x2,y2 refer to the final size.

}

UNIT _GraphIm;

INTERFACE

CONST
   { Image Mapping }
   AndPut=1;
   NormalPut=2;
   XORPut=3;
   ORPut=4;
   NoDistort=5;

PROCEDURE flip(source,dest:Word);
PROCEDURE GetImage(x1,y1,x2,y2:Integer; VAR bitmap:pointer; needmem:Boolean);
PROCEDURE PutImage(x,y:Integer; bitmap:pointer; mode:Byte;where:Word);
PROCEDURE FlipImage(bitmap:pointer;VAR newpic:pointer);
PROCEDURE scaleBitmap(VAR bitmap;x,y:Word;x1,y1,x2,y2:Word);

IMPLEMENTATION

USES
   Dos,
   _Graphic;

PROCEDURE flip(source,dest:Word);
  { This copies the entire screen at "source" to destination }
BEGIN
  asm
    push    ds
    mov     ax, [dest]
    mov     es, ax
    mov     ax, [source]
    mov     ds, ax
    XOR     si, si
    XOR     di, di
    mov     cx, 32000
    rep     movsw
    pop     ds
  END;
END;

PROCEDURE GetImage(x1,y1,x2,y2:Integer; VAR bitmap:pointer; needmem:Boolean);
{This simply stores the box designated by (x1,y1)-(x2,y2) into the pointer
   NeedMem is a boolean.  If true, then this procedure gets the necessary
   memory for the pointer.  Else, it relies on the programmer getting the mem
   in his procs. (if NeedMem is false)}
VAR l1,l2,factor:Word;
BEGIN
     factor:=3;
     IF needmem THEN
        GetMem(bitmap,(x2-x1+1)*(y2-y1+1)+4);
     MemW[0:Ofs(bitmap^)]:=(x2-x1);
     MemW[0:Ofs(bitmap^)+2]:=(y2-y1);
     FOR l1:=y1 TO y2 DO
         FOR l2:=x1 TO x2 DO BEGIN
             Inc(factor);
             Mem[0:Ofs(bitmap^)+factor]:=Mem[$A000:(l1*320)+l2];
         END;
END;

PROCEDURE PutImage(x,y:Integer; bitmap:pointer; mode:Byte;where:Word);
VAR l1,l2,factor,xdiff,ydiff:Word;
{This simply puts the image stored in a pointer back onto the screen.  It can
 XorPut it, AndPut it, NOrmal Put it, NoDistort (doesn't distort background,
 or ORPut it...}
BEGIN
     factor:=3;
     xdiff:=MemW[0:Ofs(bitmap^)];
     ydiff:=MemW[0:Ofs(bitmap^)+2];
     FOR l1:=y TO (y+ydiff) DO
         FOR l2:=x TO (x+xdiff) DO BEGIN
             Inc(factor);
             CASE mode OF
               NormalPut:Mem[where:(320*l1)+l2]:=
                                 Mem[0:Ofs(bitmap^)+factor];
               AndPut:Mem[where:(320*l1)+l2]:=Mem[where:(320*l1)+l2] AND
                                 Mem[0:Ofs(bitmap^)+factor];
               XORPut:Mem[where:(320*l1)+l2]:=Mem[where:(320*l1)+l2] XOR
                                 Mem[0:Ofs(bitmap^)+factor];
               ORPut:Mem[where:(320*l1)+l2]:=Mem[where:(320*l1)+l2] OR
                                 Mem[0:Ofs(bitmap^)+factor];
               NoDistort:IF Mem[0:Ofs(bitmap^)+factor]<>0 THEN
                                Mem[where:(320*l1)+l2]:=
                                   Mem[0:Ofs(bitmap^)+factor];
             END;
         END;
END;

PROCEDURE FlipImage(bitmap:pointer;VAR newpic:pointer);
VAR l1,l2,factor,xdiff,ydiff:Word;
{This simply puts the image stored in a pointer back onto the screen.  It can
 XorPut it, AndPut it, NOrmal Put it, NoDistort (doesn't distort background,
 or ORPut it...}
BEGIN
     factor:=3;
     xdiff:=MemW[0:Ofs(bitmap^)];
     ydiff:=MemW[0:Ofs(bitmap^)+2];
     GetMem(newpic,(xdiff+1)*(ydiff+1)+4);
     FOR l1:=0 TO ydiff DO
         FOR l2:=0 TO xdiff DO BEGIN
             Inc(factor);
             Mem[0:factor]:=
                 Mem[0:Ofs(bitmap^)+xdiff*l1+(xdiff - l2)+4];
         END;
END;

{ type
 fixed=record case boolean of
        true:(l:longint);
        false:(f:word;i:integer);
        end; }

PROCEDURE scaleBitmap(VAR bitmap;x,y:Word;x1,y1,x2,y2:Word);
TYPE
 fixed=RECORD CASE Boolean OF
        True:(l:longint);
        False:(f:Word;i:Integer);
        END;
VAR
 a,i:Word;
 sx,sy,cy,s:fixed;
 map:ARRAY[0..65521]OF Byte ABSOLUTE bitmap;
BEGIN
 sx.l:=(x*$10000)DIV Succ(x2-x1); sy.l:=(y*$10000)DIV Succ(y2-y1);
 cy.i:=Pred(y); cy.f:=$FFFF;
 WHILE cy.i>=0 DO BEGIN
  a:=y2*320+x1;
  s.l:=(cy.i*x)*$10000;
  FOR i:=x2-x1 DOWNTO 0 DO BEGIN
   Mem[$A000:a]:=map[s.i];
   Inc(a);
   Inc(s.l,sx.l);
   END;
  Dec(cy.l,sy.l); Dec(y2);
  END;
 END;

END.
